@@ -53,7 +53,6 @@ struct _CPUFreqApplet {
         PanelAppletOrient orient;
         gint              size;
 
-        GtkWidget        *container;
         GtkWidget        *box;
         GtkWidget        *icon;
         GtkWidget        *labels_box;
@@ -86,10 +85,6 @@ static void     cpufreq_applet_about_cb          (GSimpleAction *action,
                                                   GVariant      *parameter,
                                                   gpointer       user_data);
 
-static void     cpufreq_applet_pixmap_set_image  (CPUFreqApplet      *applet,
-                                                  gint                perc);
-
-static void     cpufreq_applet_setup             (CPUFreqApplet      *applet);
 static void     cpufreq_applet_update            (CPUFreqApplet      *applet,
                                                   CPUFreqMonitor     *monitor);
 static void     cpufreq_applet_refresh           (CPUFreqApplet      *applet);
@@ -106,9 +101,6 @@ static void   cpufreq_applet_get_preferred_width (GtkWidget          *widget,
                                                   gint               *natural_width);
 static void     cpufreq_applet_change_orient     (PanelApplet        *pa,
                                                   PanelAppletOrient   orient);
-static gboolean cpufreq_applet_factory           (CPUFreqApplet      *applet,
-                                                  const gchar        *iid,
-                                                  gpointer            gdata);
 
 static const gchar *const cpufreq_icons[] = {
         CPUFREQ_PIXMAPS_DIR "/cpufreq-25.png",
@@ -193,24 +185,25 @@ cpufreq_applet_init (CPUFreqApplet *applet)
 
         applet->orient = panel_applet_get_orient (PANEL_APPLET (applet));
 
+        applet->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACING);
+        gtk_container_add (GTK_CONTAINER (applet), applet->box);
+        gtk_widget_set_valign (applet->box, GTK_ALIGN_CENTER);
+        gtk_widget_show (applet->box);
+
 	switch (applet->orient) {
 	case PANEL_APPLET_ORIENT_LEFT:
 	case PANEL_APPLET_ORIENT_RIGHT:
-		applet->container = gtk_alignment_new (0.5, 0.5, 0, 0);
+		gtk_widget_set_halign (applet->box, GTK_ALIGN_CENTER);
 		break;
 	case PANEL_APPLET_ORIENT_UP:
 	case PANEL_APPLET_ORIENT_DOWN:
-		applet->container = gtk_alignment_new (0, 0.5, 0, 0);
+		gtk_widget_set_halign (applet->box, GTK_ALIGN_START);
+		break;
+	default:
+		g_assert_not_reached ();
 		break;
 	}
 
-	gtk_container_add (GTK_CONTAINER (applet), applet->container);
-	gtk_widget_show (applet->container);
-
-        applet->box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, SPACING);
-        gtk_container_add (GTK_CONTAINER (applet->container), applet->box);
-        gtk_widget_show (applet->box);
-
         applet->icon = gtk_image_new ();
         gtk_box_pack_start (GTK_BOX (applet->box), applet->icon, FALSE, FALSE, 0);
 
@@ -325,6 +318,9 @@ cpufreq_applet_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
         case PANEL_APPLET_ORIENT_DOWN:
                 size = allocation->height;
                 break;
+        default:
+                g_assert_not_reached ();
+                break;
         }
 
         if (size != applet->size) {
@@ -468,59 +464,13 @@ cpufreq_applet_get_preferred_width (GtkWidget *widget,
                 *minimum_width = *natural_width = total_width;
 }
 
-static void
-cpufreq_applet_popup_position_menu (GtkMenu  *menu,
-                                    int      *x,
-                                    int      *y,
-                                    gboolean *push_in,
-                                    gpointer  gdata)
-{
-        GtkWidget      *widget;
-        GtkRequisition  requisition;
-        GtkAllocation   allocation;
-        gint            menu_xpos;
-        gint            menu_ypos;
-
-        widget = GTK_WIDGET (gdata);
-
-        gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-
-        gdk_window_get_origin (gtk_widget_get_window (widget), &menu_xpos, &menu_ypos);
-
-	gtk_widget_get_allocation (widget, &allocation);
-
-        menu_xpos += allocation.x;
-        menu_ypos += allocation.y;
-
-        switch (panel_applet_get_orient (PANEL_APPLET (widget))) {
-        case PANEL_APPLET_ORIENT_DOWN:
-        case PANEL_APPLET_ORIENT_UP:
-                if (menu_ypos > gdk_screen_get_height (gtk_widget_get_screen (widget)) / 2)
-                        menu_ypos -= requisition.height;
-                else
-                        menu_ypos += allocation.height;
-                break;
-        case PANEL_APPLET_ORIENT_RIGHT:
-        case PANEL_APPLET_ORIENT_LEFT:
-                if (menu_xpos > gdk_screen_get_width (gtk_widget_get_screen (widget)) / 2)
-                        menu_xpos -= requisition.width;
-                else
-                        menu_xpos += allocation.width;
-                break;
-        default:
-                g_assert_not_reached ();
-        }
-
-        *x = menu_xpos;
-        *y = menu_ypos;
-        *push_in = TRUE;
-}
-
 static void
 cpufreq_applet_menu_popup (CPUFreqApplet *applet,
-                           guint32        time)
+                           GdkEvent      *event)
 {
         GtkWidget *menu;
+        GdkGravity widget_anchor;
+        GdkGravity menu_anchor;
 
         if (!cpufreq_utils_selector_is_available ())
                 return;
@@ -528,7 +478,6 @@ cpufreq_applet_menu_popup (CPUFreqApplet *applet,
         if (!applet->popup) {
                 applet->popup = cpufreq_popup_new ();
                 cpufreq_popup_set_monitor (applet->popup, applet->monitor);
-		cpufreq_popup_set_parent (applet->popup, GTK_WIDGET (applet));
         }
 
         menu = cpufreq_popup_get_menu (applet->popup);
@@ -536,10 +485,27 @@ cpufreq_applet_menu_popup (CPUFreqApplet *applet,
         if (!menu)
                 return;
 
-        gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-                        cpufreq_applet_popup_position_menu,
-                        (gpointer) applet,
-                        1, time);
+        if (applet->orient == PANEL_APPLET_ORIENT_DOWN) {
+                widget_anchor = GDK_GRAVITY_SOUTH_WEST;
+                menu_anchor = GDK_GRAVITY_NORTH_WEST;
+        } else if (applet->orient == PANEL_APPLET_ORIENT_RIGHT) {
+                widget_anchor = GDK_GRAVITY_NORTH_EAST;
+                menu_anchor = GDK_GRAVITY_NORTH_WEST;
+        } else if (applet->orient == PANEL_APPLET_ORIENT_LEFT) {
+                widget_anchor = GDK_GRAVITY_NORTH_WEST;
+                menu_anchor = GDK_GRAVITY_NORTH_EAST;
+        } else if (applet->orient == PANEL_APPLET_ORIENT_UP) {
+                widget_anchor = GDK_GRAVITY_NORTH_WEST;
+                menu_anchor = GDK_GRAVITY_SOUTH_WEST;
+        } else {
+                g_assert_not_reached ();
+        }
+
+        gtk_menu_popup_at_widget (GTK_MENU (menu),
+                                  GTK_WIDGET (applet),
+                                  widget_anchor,
+                                  menu_anchor,
+                                  event);
 }
 
 static gboolean
@@ -555,7 +521,7 @@ cpufreq_applet_button_press (GtkWidget *widget, GdkEventButton *event)
         if (event->button == 1 &&
             event->type != GDK_2BUTTON_PRESS &&
             event->type != GDK_3BUTTON_PRESS) {
-                cpufreq_applet_menu_popup (applet, event->time);
+                cpufreq_applet_menu_popup (applet, (GdkEvent *) event);
                 
                 return TRUE;
         }
@@ -577,7 +543,7 @@ cpufreq_applet_key_press (GtkWidget *widget, GdkEventKey *event)
         case GDK_KEY_Return:
         case GDK_KEY_space:
         case GDK_KEY_KP_Space:
-                cpufreq_applet_menu_popup (applet, event->time);
+                cpufreq_applet_menu_popup (applet, (GdkEvent *) event);
 
                 return TRUE;
         default:
@@ -603,12 +569,10 @@ cpufreq_applet_change_orient (PanelApplet *pa, PanelAppletOrient orient)
         if ((orient == PANEL_APPLET_ORIENT_LEFT) ||
             (orient == PANEL_APPLET_ORIENT_RIGHT)) {
                 size = allocation.width;
-		gtk_alignment_set (GTK_ALIGNMENT (applet->container),
-				   0.5, 0.5, 0, 0);
+                gtk_widget_set_halign (applet->box, GTK_ALIGN_CENTER);
         } else {
                 size = allocation.height;
-		gtk_alignment_set (GTK_ALIGNMENT (applet->container),
-				   0, 0.5, 0, 0);
+                gtk_widget_set_halign (applet->box, GTK_ALIGN_START);
         }
 
         if (size != applet->size) {
@@ -632,13 +596,12 @@ cpufreq_applet_help_cb (GSimpleAction *action,
                         GVariant      *parameter,
                         gpointer       user_data)
 {
-	CPUFreqApplet *applet = (CPUFreqApplet *) user_data;
         GError *error = NULL;
            
-	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (applet)),
-			"help:cpufreq-applet",
-			gtk_get_current_event_time (),
-			&error);
+	gtk_show_uri_on_window (NULL,
+	                        "help:cpufreq-applet",
+	                        gtk_get_current_event_time (),
+	                        &error);
 
         if (error) {
                 cpufreq_utils_display_error (_("Could not open help document"),
@@ -737,6 +700,9 @@ cpufreq_applet_update_visibility (CPUFreqApplet *applet)
                         show_freq = TRUE;
                         show_unit = TRUE;
                         break;
+                default:
+                        g_assert_not_reached ();
+                        break;
                 }
         } else {
                 show_icon = TRUE;
