@@ -19,6 +19,7 @@
  */
 
 #include "windowbuttons.h"
+#include "preferences.h"
 
 /* Prototypes */
 //static void applet_change_background (PanelApplet *, PanelAppletBackgroundType, GdkColor *, GdkPixmap *);
@@ -30,7 +31,6 @@ static void viewports_changed (WnckScreen *, WBApplet *);
 static void window_closed (WnckScreen *, WnckWindow *, WBApplet *);
 static void window_opened (WnckScreen *, WnckWindow *, WBApplet *);
 static void about_cb ( GSimpleAction *, GVariant *, gpointer );
-void properties_cb ( GSimpleAction *, GVariant *, gpointer );
 static gboolean hover_enter (GtkWidget *, GdkEventCrossing *, WBApplet *);
 static gboolean hover_leave (GtkWidget *, GdkEventCrossing *, WBApplet *);
 static gboolean button_press (GtkWidget *, GdkEventButton *, WBApplet *);
@@ -65,7 +65,7 @@ static const gchar windowbuttons_menu_items [] =
 	"</section>";
 
 static const GActionEntry windowbuttons_menu_actions [] = {
-	{ "preferences", properties_cb,  NULL, NULL, NULL },
+	{ "preferences", wb_applet_properties_cb,  NULL, NULL, NULL },
 	{ "about",       about_cb, NULL, NULL, NULL }
 };
 
@@ -73,8 +73,69 @@ WBApplet* wb_applet_new (void) {
         return g_object_new (WB_TYPE_APPLET, NULL);
 }
 
-static void wb_applet_class_init (WBAppletClass *klass) {
-	// Called before wb_applet_init()
+static void
+wb_applet_dispose (GObject *object)
+{
+  WBApplet *self;
+
+  self = WB_APPLET (object);
+
+  if (self->active_window_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->active_window_changed_id);
+      self->active_window_changed_id = 0;
+    }
+
+  if (self->viewports_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->viewports_changed_id);
+      self->viewports_changed_id = 0;
+    }
+
+  if (self->active_workspace_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->active_workspace_changed_id);
+      self->active_workspace_changed_id = 0;
+    }
+
+  if (self->window_closed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->window_closed_id);
+      self->window_closed_id = 0;
+    }
+
+  if (self->window_opened_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->window_opened_id);
+      self->window_opened_id = 0;
+    }
+
+  if (self->activewindow != NULL)
+    {
+      if (self->active_handler != 0)
+        {
+          g_signal_handler_disconnect (self->activewindow,
+                                       self->active_handler);
+          self->active_handler = 0;
+        }
+    }
+
+  G_OBJECT_CLASS (wb_applet_parent_class)->dispose (object);
+}
+
+static void
+wb_applet_class_init (WBAppletClass *self_class)
+{
+  GObjectClass *object_class;
+
+  object_class = G_OBJECT_CLASS (self_class);
+
+  object_class->dispose = wb_applet_dispose;
 }
 
 static void wb_applet_init(WBApplet *wbapplet) {
@@ -111,8 +172,6 @@ static void about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_
 		NULL
 	};
 
-	GdkPixbuf *logo = gdk_pixbuf_new_from_file (PATH_LOGO, NULL);
-
 	gtk_show_about_dialog (NULL,
 		"version",	VERSION,
 		"comments",	N_("Window buttons for your GNOME Panel."),
@@ -121,7 +180,7 @@ static void about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_
 	    "artists",	artists,
 		"documenters",	documenters,
 		"translator-credits", ("translator-credits"),
-		"logo",		logo,
+		"logo-icon-name",	"windowbuttons-applet",
 	    "website",	"http://www.gnome-look.org/content/show.php?content=103732",
    		"website-label", N_("Window Applets on Gnome-Look"),
 		NULL);
@@ -193,7 +252,9 @@ getImageState (WBButtonState button_state)
 }
 
 /* Updates the images according to preferences and the umaxed window situation */
-void updateImages (WBApplet *wbapplet) {
+static void
+updateImages (WBApplet *wbapplet)
+{
 	WnckWindow *controlledwindow;
 	gint i;
 
@@ -633,10 +694,11 @@ applet_change_orient (PanelApplet       *panelapplet,
 
 /* Do the actual applet initialization */
 static void init_wbapplet(PanelApplet *applet) {
-	WBApplet *wbapplet = g_new0 (WBApplet, 1);
+	WBApplet *wbapplet;
 	GSimpleActionGroup *action_group;
 
-	wbapplet->applet = applet;
+	wbapplet = WB_APPLET (applet);
+
 	wbapplet->settings = panel_applet_settings_new (applet, WINDOWBUTTONS_GSCHEMA);
 	wbapplet->prefs = loadPreferences(wbapplet);
 	wbapplet->activescreen = wnck_screen_get_default();
@@ -648,24 +710,40 @@ static void init_wbapplet(PanelApplet *applet) {
 	wbapplet->prefbuilder = gtk_builder_new();
 	wbapplet->box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0));
 	wbapplet->button = createButtons(wbapplet);
-	wbapplet->orient = panel_applet_get_orient(wbapplet->applet);
+	wbapplet->orient = panel_applet_get_orient(applet);
 	wbapplet->pixbufs = getPixbufs(wbapplet->prefs->images);
 
 	// Rotate & place buttons
 	placeButtons(wbapplet);
 
 	// Add box to applet
-	gtk_container_add (GTK_CONTAINER(wbapplet->applet), GTK_WIDGET(wbapplet->box));
+	gtk_container_add (GTK_CONTAINER(wbapplet), GTK_WIDGET(wbapplet->box));
 
 	// Global window tracking
-	g_signal_connect(wbapplet->activescreen, "active-window-changed", G_CALLBACK (active_window_changed), wbapplet);
-	g_signal_connect(wbapplet->activescreen, "viewports-changed", G_CALLBACK (viewports_changed), wbapplet);
-	g_signal_connect(wbapplet->activescreen, "active-workspace-changed", G_CALLBACK (active_workspace_changed), wbapplet);
-	g_signal_connect(wbapplet->activescreen, "window-closed", G_CALLBACK (window_closed), wbapplet);
-	g_signal_connect(wbapplet->activescreen, "window-opened", G_CALLBACK (window_opened), wbapplet);
+	// Global window tracking
+	wbapplet->active_window_changed_id = g_signal_connect (wbapplet->activescreen,
+	                                                       "active-window-changed",
+	                                                       G_CALLBACK (active_window_changed),
+	                                                       wbapplet);
+	wbapplet->viewports_changed_id = g_signal_connect (wbapplet->activescreen,
+	                                                   "viewports-changed",
+	                                                   G_CALLBACK (viewports_changed),
+	                                                   wbapplet);
+	wbapplet->active_workspace_changed_id = g_signal_connect (wbapplet->activescreen,
+	                                                          "active-workspace-changed",
+	                                                          G_CALLBACK (active_workspace_changed),
+	                                                          wbapplet);
+	wbapplet->window_closed_id = g_signal_connect (wbapplet->activescreen,
+	                                               "window-closed",
+	                                               G_CALLBACK (window_closed),
+	                                               wbapplet);
+	wbapplet->window_opened_id = g_signal_connect (wbapplet->activescreen,
+	                                               "window-opened",
+	                                               G_CALLBACK (window_opened),
+	                                               wbapplet);
 
-//	g_signal_connect(G_OBJECT (wbapplet->applet), "change-background", G_CALLBACK (applet_change_background), wbapplet);
-	g_signal_connect(G_OBJECT (wbapplet->applet), "change-orient", G_CALLBACK (applet_change_orient), wbapplet);
+//	g_signal_connect(G_OBJECT (wbapplet), "change-background", G_CALLBACK (applet_change_background), wbapplet);
+	g_signal_connect(G_OBJECT (wbapplet), "change-orient", G_CALLBACK (applet_change_orient), wbapplet);
 
 	// ???: Is this still necessary?
 	wbapplet->active_handler =
@@ -675,7 +753,7 @@ static void init_wbapplet(PanelApplet *applet) {
 	action_group = g_simple_action_group_new ();
 	g_action_map_add_action_entries (G_ACTION_MAP (action_group), windowbuttons_menu_actions, G_N_ELEMENTS (windowbuttons_menu_actions), wbapplet);
 	panel_applet_setup_menu (applet, windowbuttons_menu_items, action_group, GETTEXT_PACKAGE);
-	gtk_widget_insert_action_group (GTK_WIDGET (wbapplet->applet), "windowbuttons", G_ACTION_GROUP (action_group));
+	gtk_widget_insert_action_group (GTK_WIDGET (wbapplet), "windowbuttons", G_ACTION_GROUP (action_group));
 
 	toggleHidden (wbapplet);	// Properly hide or show stuff
 	updateImages (wbapplet);
@@ -694,8 +772,8 @@ void toggleHidden (WBApplet *wbapplet) {
 
 	if (!gtk_widget_get_visible(GTK_WIDGET(wbapplet->box)))
 		gtk_widget_show_all(GTK_WIDGET(wbapplet->box));
-	if (!gtk_widget_get_visible(GTK_WIDGET(wbapplet->applet)))
-		gtk_widget_show_all(GTK_WIDGET(wbapplet->applet));
+	if (!gtk_widget_get_visible(GTK_WIDGET(wbapplet)))
+		gtk_widget_show_all(GTK_WIDGET(wbapplet));
 }
 
 // Initial function that creates the applet
@@ -710,6 +788,6 @@ static gboolean windowbuttons_applet_factory (PanelApplet *applet, const gchar *
 }
 
 PANEL_APPLET_IN_PROCESS_FACTORY (APPLET_OAFIID_FACTORY,
-                                  PANEL_TYPE_APPLET,
-                                  (PanelAppletFactoryCallback) windowbuttons_applet_factory,
-                                  NULL)
+                                 WB_TYPE_APPLET,
+                                 windowbuttons_applet_factory,
+                                 NULL)
