@@ -19,9 +19,9 @@
  */
 
 #include "windowtitle.h"
+#include "preferences.h"
 
 /* Prototypes */
-//static void applet_change_background (PanelApplet *, PanelAppletBackgroundType, GdkColor *, GdkPixmap *);
 static void applet_change_orient (PanelApplet *, PanelAppletOrient, WTApplet *);
 static void active_workspace_changed (WnckScreen *, WnckWorkspace *, WTApplet *);
 static void active_window_changed (WnckScreen *, WnckWindow *, WTApplet *);
@@ -36,8 +36,6 @@ static void about_cb ( GSimpleAction *, GVariant *, gpointer );
 static WnckWindow *getRootWindow (WnckScreen *);
 static WnckWindow *getUpperMaximized (WTApplet *);
 //const gchar *getCheckBoxGConfKey (gushort);
-//void properties_cb (BonoboUIComponent *, WTApplet *, const char *);
-void properties_cb ( GSimpleAction *, GVariant *, gpointer );
 void setAlignment(WTApplet *, gdouble);
 void placeWidgets (WTApplet *);
 void reloadWidgets (WTApplet *);
@@ -49,7 +47,7 @@ WTPreferences *loadPreferences(WTApplet *);
 G_DEFINE_TYPE (WTApplet, wt_applet, PANEL_TYPE_APPLET);
 
 static GActionEntry windowtitle_menu_actions [] = {
-	{ "preferences", properties_cb,  NULL, NULL, NULL },
+	{ "preferences", wt_applet_properties_cb,  NULL, NULL, NULL },
 	{ "about",       about_cb, NULL, NULL, NULL }
 };
 
@@ -69,8 +67,83 @@ WTApplet* wt_applet_new (void) {
         return g_object_new (WT_TYPE_APPLET, NULL);
 }
 
-static void wt_applet_class_init (WTAppletClass *klass) {
-	// Not required
+static void
+wt_applet_dispose (GObject *object)
+{
+  WTApplet *self;
+
+  self = WT_APPLET (object);
+
+  if (self->active_window_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->active_window_changed_id);
+      self->active_window_changed_id = 0;
+    }
+
+  if (self->viewports_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->viewports_changed_id);
+      self->viewports_changed_id = 0;
+    }
+
+  if (self->active_workspace_changed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->active_workspace_changed_id);
+      self->active_workspace_changed_id = 0;
+    }
+
+  if (self->window_closed_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->window_closed_id);
+      self->window_closed_id = 0;
+    }
+
+  if (self->window_opened_id != 0)
+    {
+      g_signal_handler_disconnect (self->activescreen,
+                                   self->window_opened_id);
+      self->window_opened_id = 0;
+    }
+
+  if (self->activewindow != NULL)
+    {
+      if (self->active_handler_state != 0)
+        {
+          g_signal_handler_disconnect (self->activewindow,
+                                       self->active_handler_state);
+          self->active_handler_state = 0;
+        }
+
+      if (self->active_handler_name != 0)
+        {
+          g_signal_handler_disconnect (self->activewindow,
+                                       self->active_handler_name);
+          self->active_handler_name = 0;
+        }
+
+      if (self->active_handler_icon != 0)
+        {
+          g_signal_handler_disconnect (self->activewindow,
+                                       self->active_handler_icon);
+          self->active_handler_icon = 0;
+        }
+    }
+
+  G_OBJECT_CLASS (wt_applet_parent_class)->dispose (object);
+}
+
+static void
+wt_applet_class_init (WTAppletClass *self_class)
+{
+  GObjectClass *object_class;
+
+  object_class = G_OBJECT_CLASS (self_class);
+
+  object_class->dispose = wt_applet_dispose;
 }
 
 static void wt_applet_init(WTApplet *wtapplet) {
@@ -98,8 +171,6 @@ static void about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_
 		NULL
 	};
 
-	GdkPixbuf *logo = gdk_pixbuf_new_from_file (PATH_LOGO, NULL);
-
 	gtk_show_about_dialog (NULL,
 		"version",	VERSION,
 		"comments",	N_("Window title for your GNOME Panel."),
@@ -108,7 +179,7 @@ static void about_cb (GSimpleAction *action, GVariant *parameter, gpointer user_
 	    "artists",	artists,
 		"documenters",	documenters,
 		"translator-credits",	("translator-credits"),
-		"logo",		logo,
+		"logo-icon-name",	"windowtitle-applet",
    	    "website",	"http://www.gnome-look.org/content/show.php?content=103732",
 		"website-label", N_("Window Applets on Gnome-Look"),
 		NULL);
@@ -175,9 +246,12 @@ static WnckWindow *getUpperMaximized (WTApplet *wtapplet) {
 
 // Updates the images according to preferences and the window situation
 // Warning! This function is called very often, so it should only do the most necessary things!
-void updateTitle(WTApplet *wtapplet) {
+static void
+updateTitle (WTApplet *wtapplet)
+{
 	WnckWindow *controlledwindow;
-	gchar *title_text, *title_color, *title_font;
+	const char *title_text, *title_color, *title_font;
+	GString *markup;
 	GdkPixbuf *icon_pixbuf;
 
 	if (wtapplet->prefs->only_maximized) {
@@ -202,7 +276,7 @@ void updateTitle(WTApplet *wtapplet) {
 		}
 	} else {
 		icon_pixbuf = wnck_window_get_icon(controlledwindow); // This only returns a pointer - it SHOULDN'T be unrefed!
-		title_text = (gchar*)wnck_window_get_name(controlledwindow);
+		title_text = wnck_window_get_name(controlledwindow);
 	}
 
 	// TODO: we need the default font to somehow be the same in both modes
@@ -221,7 +295,7 @@ void updateTitle(WTApplet *wtapplet) {
 		// automatic (non-custom) style
 		if (controlledwindow == wtapplet->activewindow) {
 			// window focused
-			title_color = wtapplet->panel_color_fg;
+			title_color = "";
 			title_font = "";
 		} else {
 			// window unfocused
@@ -236,10 +310,19 @@ void updateTitle(WTApplet *wtapplet) {
 		gtk_widget_set_tooltip_text (GTK_WIDGET(wtapplet->title), title_text);
 	}
 
-	title_text = g_markup_printf_escaped("<span font=\"%s\" color=\"%s\">%s</span>", title_font, title_color, title_text);
+	markup = g_string_new ("<span");
+
+	if (title_font != NULL && *title_font != '\0')
+		g_string_append_printf (markup, " font=\"%s\"", title_font);
+
+	if (title_color != NULL && *title_color != '\0')
+		g_string_append_printf (markup, " color=\"%s\"", title_color);
+
+	g_string_append_printf (markup, ">%s</span>", title_text);
+
 	// Apply markup to label widget
-	gtk_label_set_markup(GTK_LABEL(wtapplet->title), title_text);
-	g_free(title_text);
+	gtk_label_set_markup(GTK_LABEL(wtapplet->title), markup->str);
+	g_string_free (markup, TRUE);
 
 	if (icon_pixbuf == NULL) {
 		gtk_image_clear(wtapplet->icon);
@@ -261,13 +344,15 @@ void updateTitle(WTApplet *wtapplet) {
 }
 
 /* Expand/unexpand applet according to preferences */
-void toggleExpand(WTApplet *wtapplet) {
+static void
+toggleExpand (WTApplet *wtapplet)
+{
 	if (wtapplet->prefs->expand_applet) {
-		panel_applet_set_flags (wtapplet->applet, PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_EXPAND_MAJOR);
+		panel_applet_set_flags (PANEL_APPLET (wtapplet), PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_EXPAND_MAJOR);
 	} else {
 		// We must have a handle due to bug https://bugzilla.gnome.org/show_bug.cgi?id=556355
-		// panel_applet_set_flags (wtapplet->applet, PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_EXPAND_MAJOR | PANEL_APPLET_HAS_HANDLE);
-		panel_applet_set_flags (wtapplet->applet, PANEL_APPLET_EXPAND_MINOR);
+		// panel_applet_set_flags (PANEL_APPLET (wtapplet), PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_EXPAND_MAJOR | PANEL_APPLET_HAS_HANDLE);
+		panel_applet_set_flags (PANEL_APPLET (wtapplet), PANEL_APPLET_EXPAND_MINOR);
 	}
 	reloadWidgets(wtapplet);
 	setAlignment(wtapplet, (gdouble)wtapplet->prefs->alignment);
@@ -293,37 +378,8 @@ void toggleHidden (WTApplet *wtapplet) {
 		gtk_widget_show_all(GTK_WIDGET(wtapplet->eb_title));
 	if (!gtk_widget_get_visible(GTK_WIDGET(wtapplet->box)))
 		gtk_widget_show_all(GTK_WIDGET(wtapplet->box));
-	if (!gtk_widget_get_visible(GTK_WIDGET(wtapplet->applet)))
-		gtk_widget_show_all(GTK_WIDGET(wtapplet->applet));
-}
-
-/* Called when panel background is changed */
-static void applet_change_background (PanelApplet		*applet,
-				     				 cairo_pattern_t	*pattern,
-									 WTApplet			*wtapplet)
-{
-	if (wtapplet->panel_color_fg)
-		g_free(wtapplet->panel_color_fg);
-
-	// Look up the default text color in the theme, use a default if it's not defined
-	// This way is deprecated
-	GdkColor color;
-	GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(applet));
-	if (!gtk_style_lookup_color(style, "dark_fg_color", &color))	// check if dark_fg_color is set otherwise...
-		if (!gtk_style_lookup_color(style, "fg_color", &color))		// ... check if fg_color is set otherwise...
-			gdk_color_parse("#808080", &color);						// ... set universally acceptable color #808080
-	wtapplet->panel_color_fg = gdk_color_to_string(&color);
-
-	/*
-	GdkRGBA rgba;
-	GtkStyleContext *stylecontext = gtk_widget_get_style_context(GTK_WIDGET(applet));
-	if (!gtk_style_context_lookup_color(stylecontext, "dark_fg_color", &rgba))
-		if (!gtk_style_context_lookup_color(stylecontext, "fg_color", &rgba))
-			gdk_rgba_parse(&rgba, "#808080");
-	wtapplet->panel_color_fg = gdk_rgba_to_string(&rgba); // This does not produce the right syntax (gtk_label_set_markup fails)
-	*/
-
-	updateTitle(wtapplet); // We need to redraw the title using the new colors
+	if (!gtk_widget_get_visible(GTK_WIDGET(wtapplet)))
+		gtk_widget_show_all(GTK_WIDGET(wtapplet));
 }
 
 /* Triggered when a different panel orientation is detected */
@@ -339,19 +395,6 @@ static void applet_change_orient (PanelApplet *panelapplet,
 	}
 }
 
-/* (Supposedly) tiggered when panel size changes */
-static void applet_change_pixel_size (PanelApplet	*applet,
-									  gint			size,
-									  WTApplet		*wtapplet)
-{
-	if (wtapplet->size == size)
-		return;
-
-	wtapplet->size = size;
-
-	updateTitle(wtapplet);
-}
-
 /*
 static void applet_title_size_request (GtkWidget *widget,
                                        GtkRequisition *requisition,
@@ -666,9 +709,10 @@ void setAlignment (WTApplet *wtapplet, gdouble alignment) {
 
 /* Do the actual applet initialization */
 static void init_wtapplet (PanelApplet *applet) {
-	WTApplet *wtapplet = g_new0 (WTApplet, 1);
+	WTApplet *wtapplet;
+
+	wtapplet = WT_APPLET (applet);
 
-	wtapplet->applet = applet;
 	wtapplet->settings = panel_applet_settings_new (applet, WINDOWTITLE_GSCHEMA);
 	wtapplet->prefs = loadPreferences(wtapplet);
 	wtapplet->activescreen = wnck_screen_get_default();
@@ -683,7 +727,7 @@ static void init_wtapplet (PanelApplet *applet) {
 	wtapplet->title = GTK_LABEL(gtk_label_new(NULL));
 	wtapplet->eb_icon = GTK_EVENT_BOX(gtk_event_box_new());
 	wtapplet->eb_title = GTK_EVENT_BOX(gtk_event_box_new());
-	wtapplet->orient = panel_applet_get_orient(wtapplet->applet);
+	wtapplet->orient = panel_applet_get_orient(applet);
 	wtapplet->size_hints = g_new(gint,2);
 
 	// Widgets to eventboxes, eventboxes to box
@@ -699,25 +743,37 @@ static void init_wtapplet (PanelApplet *applet) {
 	placeWidgets(wtapplet);
 
 	// Add box to applet
-	gtk_container_add (GTK_CONTAINER(wtapplet->applet), GTK_WIDGET(wtapplet->box));
+	gtk_container_add (GTK_CONTAINER(wtapplet), GTK_WIDGET(wtapplet->box));
 
 	// Set event handling (icon & title clicks)
 	g_signal_connect(G_OBJECT (wtapplet->eb_icon), "button-press-event", G_CALLBACK (icon_clicked), wtapplet);
 	g_signal_connect(G_OBJECT (wtapplet->eb_title), "button-press-event", G_CALLBACK (title_clicked), wtapplet);
 
 	// Global window tracking
-	g_signal_connect(wtapplet->activescreen, "active-window-changed", G_CALLBACK (active_window_changed), wtapplet); // <-- this thing is crashing with compiz !!!
-	g_signal_connect(wtapplet->activescreen, "viewports-changed", G_CALLBACK (viewports_changed), wtapplet);
-	g_signal_connect(wtapplet->activescreen, "active-workspace-changed", G_CALLBACK (active_workspace_changed), wtapplet);
-	g_signal_connect(wtapplet->activescreen, "window-closed", G_CALLBACK (window_closed), wtapplet);
-	g_signal_connect(wtapplet->activescreen, "window-opened", G_CALLBACK (window_opened), wtapplet);
+	wtapplet->active_window_changed_id = g_signal_connect (wtapplet->activescreen,
+	                                                       "active-window-changed",
+	                                                       G_CALLBACK (active_window_changed),
+	                                                       wtapplet); // <-- this thing is crashing with compiz !!!
+	wtapplet->viewports_changed_id = g_signal_connect (wtapplet->activescreen,
+	                                                   "viewports-changed",
+	                                                   G_CALLBACK (viewports_changed),
+	                                                   wtapplet);
+	wtapplet->active_workspace_changed_id = g_signal_connect (wtapplet->activescreen,
+	                                                          "active-workspace-changed",
+	                                                          G_CALLBACK (active_workspace_changed),
+	                                                          wtapplet);
+	wtapplet->window_closed_id = g_signal_connect (wtapplet->activescreen,
+	                                               "window-closed",
+	                                               G_CALLBACK (window_closed),
+	                                               wtapplet);
+	wtapplet->window_opened_id = g_signal_connect (wtapplet->activescreen,
+	                                               "window-opened",
+	                                               G_CALLBACK (window_opened),
+	                                               wtapplet);
 
 	// g_signal_connect(G_OBJECT (wtapplet->title), "size-request", G_CALLBACK (applet_title_size_request), wtapplet);
-	g_signal_connect(G_OBJECT (wtapplet->applet), "size-allocate", G_CALLBACK (applet_size_allocate), wtapplet);
-
-	g_signal_connect(G_OBJECT (wtapplet->applet), "change-background", G_CALLBACK (applet_change_background), wtapplet);
-	g_signal_connect(G_OBJECT (wtapplet->applet), "change-orient", G_CALLBACK (applet_change_orient), wtapplet);
-	g_signal_connect(G_OBJECT (wtapplet->applet), "change-size", G_CALLBACK (applet_change_pixel_size), wtapplet);
+	g_signal_connect(G_OBJECT (wtapplet), "size-allocate", G_CALLBACK (applet_size_allocate), wtapplet);
+	g_signal_connect(G_OBJECT (wtapplet), "change-orient", G_CALLBACK (applet_change_orient), wtapplet);
 
 	// Track active window changes
 	wtapplet->active_handler_state =
@@ -732,7 +788,7 @@ static void init_wtapplet (PanelApplet *applet) {
 	GSimpleActionGroup *action_group = g_simple_action_group_new ();
 	g_action_map_add_action_entries (G_ACTION_MAP (action_group), windowtitle_menu_actions, G_N_ELEMENTS (windowtitle_menu_actions), wtapplet);
 	panel_applet_setup_menu (applet, windowtitle_menu_items, action_group, GETTEXT_PACKAGE);
-	gtk_widget_insert_action_group (GTK_WIDGET (wtapplet->applet), "windowtitle", G_ACTION_GROUP (action_group));
+	gtk_widget_insert_action_group (GTK_WIDGET (wtapplet), "windowtitle", G_ACTION_GROUP (action_group));
 
 	toggleExpand  (wtapplet);
 	toggleHidden  (wtapplet);	// Properly hide or show stuff
@@ -751,6 +807,6 @@ static gboolean windowtitle_applet_factory (PanelApplet *applet, const gchar *ii
 }
 
 PANEL_APPLET_IN_PROCESS_FACTORY (APPLET_OAFIID_FACTORY,
-                                  PANEL_TYPE_APPLET,
-                                  (PanelAppletFactoryCallback) windowtitle_applet_factory,
-                                  NULL)
+                                 WT_TYPE_APPLET,
+                                 windowtitle_applet_factory,
+                                 NULL)
